# import various functions here to allow for easy import
# from the biogl module (e.g. from biogl import {func})

from biogl.fasta_parse import fasta_parse
from biogl.GxfParse import GxfParse
from biogl.rev_comp import rev_comp
from biogl.translate import translate
from biogl.flex_open import flex_open
from biogl.random_seq import random_seq
from biogl.get_runtime import get_runtime
from biogl.byte_convert import byte_convert
from biogl.label_info import label_info
from biogl.is_fasta import is_fasta
from biogl.recursive_items import recursive_items
from biogl.window import window
